ONE_OR_THREE_ARGS_REQUIRED = (
    'Передано некоректное количество аргументов. '
    'Их должно быть 1 или 3.'
)
TWO_ARGS_REQUIRED = (
    'Передано некоректное количество аргументов. '
    'Их должно быть 2.'
)
TWO_ARGS_AND_TRAPEZOID_H_REQUIRED = (
    'Передано некоректное количество аргументов.'
    'Должны быть переданы 2 размера:'
    'длинна оснований трапеции,'
    'а также её высота - параметр trapezoid_h.'
)
UNKNOWN_ERROR = (
    'Неизвестная ошибка выполнения программы.'
)
